@@ -27,9 +27,9 @@ fexpt = qutip.expect(a.dag() * a, final_state)
 
 tlist = np.linspace(0, 50, 100)
 # monte-carlo
-mcdata = qutip.mcsolve(H, psi0, tlist, c_op_list, [a.dag() * a], ntraj=100)
+mcdata = qutip.mcsolve(H, psi0, tlist, c_op_list, e_ops=[a.dag() * a], ntraj=100)
 # master eq.
-medata = qutip.mesolve(H, psi0, tlist, c_op_list, [a.dag() * a])
+medata = qutip.mesolve(H, psi0, tlist, c_op_list, e_ops=[a.dag() * a])
 
 plt.plot(tlist, mcdata.expect[0], tlist, medata.expect[0], lw=2)
 # plot steady-state expt. value as horizontal line (should be = 2)
