@@ -9,8 +9,11 @@ __all__ = [
 
 import numpy as np
 from functools import partial
+from typing import TypeVar, overload
+
 from .operators import qeye
 from .qobj import Qobj
+from .cy.qobjevo import QobjEvo
 from .superoperator import operator_to_vector, reshuffle
 from .dimensions import (
     flatten, enumerate_flat, unflatten, deep_remove, dims_to_tensor_shape,
@@ -18,6 +21,7 @@ from .dimensions import (
 )
 from . import data as _data
 from .. import settings
+from ..typing import LayerType
 
 
 class _reverse_partial_tensor:
@@ -29,7 +33,13 @@ class _reverse_partial_tensor:
         return tensor(op, self.right)
 
 
-def tensor(*args):
+@overload
+def tensor(*args: Qobj) -> Qobj: ...
+
+@overload
+def tensor(*args: Qobj | QobjEvo) -> QobjEvo: ...
+
+def tensor(*args: Qobj | QobjEvo) -> Qobj | QobjEvo:
     """Calculates the tensor product of input operators.
 
     Parameters
@@ -106,7 +116,13 @@ shape = [4, 4], type = oper, isHerm = True
                 copy=False)
 
 
-def super_tensor(*args):
+@overload
+def super_tensor(*args: Qobj) -> Qobj: ...
+
+@overload
+def super_tensor(*args: Qobj | QobjEvo) -> QobjEvo: ...
+
+def super_tensor(*args: Qobj | QobjEvo) -> Qobj | QobjEvo:
     """
     Calculate the tensor product of input superoperators, by tensoring together
     the underlying Hilbert spaces on which each vectorized operator acts.
@@ -174,6 +190,12 @@ def _isbralike(q):
     return q.isbra or q.isoperbra
 
 
+@overload
+def composite(*args: Qobj) -> Qobj: ...
+
+@overload
+def composite(*args: Qobj | QobjEvo) -> QobjEvo: ...
+
 def composite(*args):
     """
     Given two or more operators, kets or bras, returns the Qobj
@@ -245,13 +267,18 @@ def _tensor_contract_dense(arr, *pairs):
     return arr
 
 
-def tensor_swap(q_oper, *pairs):
+def tensor_swap(q_oper: Qobj, *pairs: tuple[int, int]) -> Qobj:
     """Transposes one or more pairs of indices of a Qobj.
-    Note that this uses dense representations and thus
-    should *not* be used for very large Qobjs.
+
+    .. note::
+
+        Note that this uses dense representations and thus
+        should *not* be used for very large Qobjs.
 
     Parameters
     ----------
+    q_oper : Qobj
+        Operator to swap dims.
 
     pairs : tuple
         One or more tuples ``(i, j)`` indicating that the
@@ -284,10 +311,13 @@ def tensor_swap(q_oper, *pairs):
     return Qobj(data, dims=dims, superrep=q_oper.superrep, copy=False)
 
 
-def tensor_contract(qobj, *pairs):
+def tensor_contract(qobj: Qobj, *pairs: tuple[int, int]) -> Qobj:
     """Contracts a qobj along one or more index pairs.
-    Note that this uses dense representations and thus
-    should *not* be used for very large Qobjs.
+
+    .. note::
+
+        Note that this uses dense representations and thus
+        should *not* be used for very large Qobjs.
 
     Parameters
     ----------
@@ -414,7 +444,15 @@ def _targets_to_list(targets, oper=None, N=None):
     return targets
 
 
-def expand_operator(oper, dims, targets, dtype=None):
+QobjOrQobjEvo = TypeVar("QobjOrQobjEvo", Qobj, QobjEvo)
+
+
+def expand_operator(
+    oper: QobjOrQobjEvo,
+    dims: list[int],
+    targets: int,
+    dtype: LayerType = None
+) -> QobjOrQobjEvo:
     """
     Expand an operator to one that acts on a system with desired dimensions.
     e.g.
