@@ -1,12 +1,29 @@
+# Required for Sphinx to follow autodoc_type_aliases
+from __future__ import annotations
+
 __all__ = ['krylovsolve']
 
-from .. import QobjEvo
+from .. import QobjEvo, Qobj
 from .sesolve import SESolver
+from .result import Result
+from .solver_base import _kwargs_migration
+from numpy.typing import ArrayLike
+from typing import Any, Callable
 
 
 def krylovsolve(
-    H, psi0, tlist, krylov_dim, e_ops=None, args=None, options=None
-):
+    H: Qobj,
+    psi0: Qobj,
+    tlist: ArrayLike,
+    krylov_dim: int,
+    _e_ops = None,
+    _args = None,
+    _options = None,
+    *,
+    e_ops: dict[Any, Qobj | QobjEvo | Callable[[float, Qobj], Any]] = None,
+    args: dict[str, Any] = None,
+    options: dict[str, Any] = None,
+) -> Result:
     """
     Schrodinger equation evolution of a state vector for time independent
     Hamiltonians using Krylov method.
@@ -95,6 +112,9 @@ def krylovsolve(
         vectors or density matrices corresponding to the times in ``tlist`` [if
         ``e_ops`` is an empty list of ``store_states=True`` in options].
     """
+    e_ops = _kwargs_migration(_e_ops, e_ops, "e_ops")
+    args = _kwargs_migration(_args, args, "args")
+    options = _kwargs_migration(_options, options, "options")
     H = QobjEvo(H, args=args, tlist=tlist)
     options = options or {}
     options["method"] = "krylov"
