@@ -1,5 +1,11 @@
+# Required for Sphinx to follow autodoc_type_aliases
+from __future__ import annotations
+
 __all__ = ['Solver']
 
+from numpy.typing import ArrayLike
+from numbers import Number
+from typing import Any, Callable
 from .. import Qobj, QobjEvo, ket2dm
 from .options import _SolverOptions
 from ..core import stack_columns, unstack_columns
@@ -8,6 +14,7 @@ from .result import Result
 from .integrator import Integrator
 from ..ui.progressbar import progress_bars
 from ._feedback import _ExpectFeedback
+from ..typing import EopsLike
 from time import time
 import warnings
 import numpy as np
@@ -90,7 +97,7 @@ class Solver:
             # This is herm flag take for granted that the liouvillian keep
             # hermiticity.  But we do not check user passed super operator for
             # anything other than dimensions.
-            'isherm': state.isherm and not (self.rhs.dims == state.dims)
+            'isherm': not (self.rhs.dims == state.dims) and state._isherm,
         }
         if state.isket:
             norm = state.norm()
@@ -131,7 +138,14 @@ class Solver:
 
         return state
 
-    def run(self, state0, tlist, *, args=None, e_ops=None):
+    def run(
+        self,
+        state0: Qobj,
+        tlist: ArrayLike,
+        *,
+        e_ops: EopsLike | list[EopsLike] | dict[Any, EopsLike] = None,
+        args: dict[str, Any] = None,
+    ) -> Result:
         """
         Do the evolution of the Quantum system.
 
@@ -151,12 +165,12 @@ class Solver:
             evolution. Each times of the list must be increasing, but does not
             need to be uniformy distributed.
 
-        args : dict, optional {None}
+        args : dict, optional
             Change the ``args`` of the rhs for the evolution.
 
-        e_ops : list {None}
-            List of Qobj, QobjEvo or callable to compute the expectation
-            values. Function[s] must have the signature
+        e_ops : Qobj, QobjEvo, callable, list, or dict optional
+            Single, list or dict of Qobj, QobjEvo or callable to compute the
+            expectation values. Function[s] must have the signature
             f(t : float, state : Qobj) -> expect.
 
         Returns
@@ -190,7 +204,7 @@ class Solver:
         # stats.update(_integrator.stats)
         return results
 
-    def start(self, state0, t0):
+    def start(self, state0: Qobj, t0: Number) -> None:
         """
         Set the initial state and time for a step evolution.
 
@@ -206,7 +220,13 @@ class Solver:
         self._integrator.set_state(t0, self._prepare_state(state0))
         self.stats["preparation time"] += time() - _time_start
 
-    def step(self, t, *, args=None, copy=True):
+    def step(
+        self,
+        t: Number,
+        *,
+        args: dict[str, Any] = None,
+        copy: bool = True
+    ) -> Qobj:
         """
         Evolve the state to ``t`` and return the state as a :obj:`.Qobj`.
 
@@ -263,7 +283,7 @@ class Solver:
         return self.rhs.dims[0]
 
     @property
-    def options(self):
+    def options(self) -> dict[str, Any]:
         """
         method: str
             Which ordinary differential equation integration method to use.
@@ -303,7 +323,7 @@ class Solver:
         return included_options, extra_options
 
     @options.setter
-    def options(self, new_options):
+    def options(self, new_options: dict[str, Any]):
         if not hasattr(self, "_options"):
             self._options = {}
         if new_options is None:
@@ -424,7 +444,7 @@ class Solver:
         cls._avail_integrators[key] = integrator
 
     @classmethod
-    def ExpectFeedback(cls, operator, default=0.):
+    def ExpectFeedback(cls, operator: Qobj | QobjEvo, default: Any = 0.):
         """
         Expectation value of the instantaneous state of the evolution to be
         used by a time-dependent operator.
@@ -549,3 +569,13 @@ def _solver_deprecation(kwargs, options, solver="me"):
     if kwargs:
         raise TypeError(f"unexpected keyword argument {kwargs.keys()}")
     return options
+
+
+def _kwargs_migration(position, keyword, name):
+    if position is not None:
+        warnings.warn(
+            f"{name} will be keyword only from qutip 5.3 for all solver",
+            FutureWarning
+        )
+        return position
+    return keyword
