@@ -1,5 +1,5 @@
-from qutip.about import about
-from qutip import settings as qset
+from .about import about
+from .settings import settings as qset
 
 def run(full=False):
     """
@@ -14,13 +14,13 @@ def run(full=False):
     # Call about to get all version info printed with tests
     about()
     import pytest
-    real_num_cpu = qset.num_cpus
-    real_thresh = qset.openmp_thresh
-    if qset.has_openmp:
+    # real_num_cpu = qset.num_cpus
+    # real_thresh = qset.openmp_thresh
+    # if qset.has_openmp:
         # For travis which VMs have only 1 cpu.
         # Make sure the openmp version of the functions are tested.
-        qset.num_cpus = 2
-        qset.openmp_thresh = 100
+    #     qset.num_cpus = 2
+    #     qset.openmp_thresh = 100
 
     test_options = ["--verbosity=1", "--disable-pytest-warnings", "--pyargs"]
     if not full:
@@ -29,6 +29,6 @@ def run(full=False):
     # runs tests in qutip.tests module only
 
     # Restore previous settings
-    if qset.has_openmp:
-        qset.num_cpus = real_num_cpu
-        qset.openmp_thresh = real_thresh
+    # if qset.has_openmp:
+    #     qset.num_cpus = real_num_cpu
+    #     qset.openmp_thresh = real_thresh
