@@ -1,10 +1,8 @@
 import pytest
 import functools
 import os
-import sys
 import tempfile
 import numpy as np
-from qutip.utilities import _version2int
 
 
 def _add_repeats_if_marked(metafunc):
@@ -29,13 +27,8 @@ def _skip_cython_tests_if_unavailable(item):
     if item.get_closest_marker("requires_cython"):
         # importorskip rather than mark.skipif because this way we get pytest's
         # version-handling semantics.
-        _Cython = pytest.importorskip('Cython', minversion='0.14')
-        # importorskip does not have maxversion
-        if _version2int(_Cython.__version__) >= _version2int("3.0.0"):
-            pytest.skip("cython 3.0.0 not supported")
-            return
-        if _version2int(sys.version.split()[0]) >= _version2int("3.12.0"):
-            pytest.skip("Cython not available at runtime with python >=3.12.")
+        pytest.importorskip('Cython', minversion='0.14')
+        pytest.importorskip('filelock')
 
 
 @pytest.hookimpl(trylast=True)
@@ -107,7 +100,7 @@ def _patched_build_err_msg(arrays, err_msg, header='Items are not equal:',
             except Exception as exc:
                 r = '[repr failed for <{}>: {}]'.format(type(a).__name__, exc)
             # [diff] The original truncates the output to 3 lines here.
-            msg.append(' %s: %s' % (names[i], r))
+            msg.append(f' {names[i]}: {r}')
     return '\n'.join(msg)
 
 
