@@ -57,9 +57,11 @@ class TestSeSolve():
         options = {"progress_bar": None}
 
         if unitary_op is None:
-            output = sesolve(H, psi0, self.tlist,
-                             [qutip.sigmax(), qutip.sigmay(), qutip.sigmaz()],
-                             args=self.args, options=options)
+            output = sesolve(
+                H, psi0, self.tlist,
+                e_ops=[qutip.sigmax(), qutip.sigmay(), qutip.sigmaz()],
+                args=self.args, options=options
+            )
             sx, sy, sz = output.expect[0], output.expect[1], output.expect[2]
         else:
             output = sesolve(H, unitary_op, self.tlist, args=self.args,
@@ -125,9 +127,11 @@ class TestSeSolve():
         H = [[self.H1, 'exp(-alpha*t)']]
 
         if unitary_op is None:
-            output = sesolve(H, psi0, self.tlist,
-                             [qutip.sigmax(), qutip.sigmay(), qutip.sigmaz()],
-                             args=self.args, options=options)
+            output = sesolve(
+                H, psi0, self.tlist,
+                e_ops=[qutip.sigmax(), qutip.sigmay(), qutip.sigmaz()],
+                args=self.args, options=options
+            )
             sx, sy, sz = output.expect[0], output.expect[1], output.expect[2]
         else:
             output = sesolve(H, unitary_op, self.tlist,
@@ -179,10 +183,11 @@ class TestSeSolve():
             "normalize_output": normalize,
             "progress_bar": None
         }
-        out_s = sesolve(H, psi0, self.tlist, [qutip.sigmax(),
-                                              qutip.sigmay(),
-                                              qutip.sigmaz()],
-                        options=options, args=args)
+        out_s = sesolve(
+            H, psi0, self.tlist,
+            e_ops=[qutip.sigmax(), qutip.sigmay(), qutip.sigmaz()],
+            options=options, args=args
+        )
         xs, ys, zs = out_s.expect[0], out_s.expect[1], out_s.expect[2]
         xss = [qutip.expect(qutip.sigmax(), U) for U in out_s.states]
         yss = [qutip.expect(qutip.sigmay(), U) for U in out_s.states]
